/* $Id: queue.c,v 1.10 2007-05-24 16:08:55 kristaps Exp $ */
/*
Copyright (c) 2007 Kristaps Dzonsons
Latvijas Universitaates Matemaatikas un Informaatikas Instituuts
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
* Neither the name of the copyright holders nor the names of
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/param.h>
#include <sys/queue.h>

#include <assert.h>
#include <err.h>
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "nlpcrawl.h"


/*	$OpenBSD: queue.h,v 1.31 2005/11/25 08:06:25 otto Exp $	 */
/*	$NetBSD: queue.h,v 1.11 1996/05/16 05:17:14 mycroft Exp $ */

/*
 * Copyright (c) 1991, 1993
 *	The Regents of the University of California.  All rights
 *	reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	@(#)queue.h	8.5 (Berkeley) 8/20/94
 */
#ifndef LIST_FIRST
# define LIST_FIRST(head)	((head)->lh_first)
#endif
#ifndef LIST_END
# define LIST_END(head)		NULL
#endif
#ifndef LIST_EMPTY
# define LIST_EMPTY(head)	(LIST_FIRST(head) == LIST_END(head))
#endif


struct node {
	LIST_ENTRY(node) node_next;
	void		*data;
};


/*
 * Note that this is an extension to LIST_HEAD that defines the last
 * element, as well (in order to have queue functionality).
 */
struct nodequeue {
	struct node 	*lh_first;
	struct node 	*last;
};


void
q_new(void **cookie)
{
	struct nodequeue *q;

	assert(NULL != cookie);
	if (NULL == (q = malloc(sizeof(struct nodequeue))))
		die("%s: fail: allocate", __func__);

	LIST_INIT(q);
	q->last = NULL;

	*cookie = q;
}


void
q_free(void **cookie)
{
	assert(NULL != cookie);

	if (NULL == *cookie)
		return;
	while (NULL != q_rem(cookie)) 
		/* Do nothing. */ ;
	free(*cookie);
	*cookie = NULL;
}


int
q_empty(void **cookie)
{
	assert(NULL != cookie);
	return(LIST_EMPTY(*(struct nodequeue **)cookie));
}


void
q_add(void **cookie, void *p)
{
	struct node	*n, *last;

	assert(NULL != p);
	assert(NULL != cookie);
	assert(NULL != *cookie);

	if (NULL == (n = malloc(sizeof(struct node))))
		die("%s: fail: allocate", __func__);
	n->data = p;

	if (NULL == (last = (*(struct nodequeue **)cookie)->last)) {
		LIST_INSERT_HEAD(*(struct nodequeue **)cookie, 
				n, node_next);
		last = LIST_FIRST(*(struct nodequeue **)cookie);
	} else
		LIST_INSERT_AFTER(last, n, node_next);
}


void *
q_rem(void **cookie)
{
	struct node	*n;
	void		*p;

	assert(NULL != cookie);
	assert(NULL != *cookie);

	if (NULL == (n = LIST_FIRST(*(struct nodequeue **)cookie))) 
		return(NULL);

	LIST_REMOVE(n, node_next);
	if (n == (*(struct nodequeue **)cookie)->last)
		(*(struct nodequeue **)cookie)->last = NULL;

	p = n->data;
	free(n);

	return(p);
}


const void *
q_head(void **cookie)
{
	struct node	*n;

	assert(NULL != cookie);
	assert(NULL != *cookie);

	if (NULL == (n = LIST_FIRST(*(struct nodequeue **)cookie))) 
		return(NULL);
	return(n->data);
}
