/* $Id: util.c,v 1.30 2007-05-23 15:04:28 kristaps Exp $ */
/*
Copyright (c) 2007 Kristaps Dzonsons
Latvijas Universitaates Matemaatikas un Informaatikas Instituuts
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
* Neither the name of the copyright holders nor the names of
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/param.h>
#include <sys/stat.h>

#include <assert.h>
#include <err.h>
#include <errno.h>
#include <limits.h>
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "nlpcrawl.h"



uint64_t
time_uint64(void)
{
	time_t		tm;
	time_s(tm);
	return((uint64_t)tm);
}


int
parse_hex(const char *from, int *to, int max, int min)
{
	long 		lval;
	char		*ep;

	assert(NULL != from);
	assert(NULL != to);
	
	errno = 0;
	lval = strtol(from, &ep, 16);
	if (from[0] == '\0' || *ep != '\0') {
		return(0);
	}
	if ((errno == ERANGE && (lval == LONG_MAX || lval == LONG_MIN)) 
			|| (lval > max || lval < min)) {
		dddbg("lval = %ld, max = %d, min = %d", lval, max, min);
		return(0);
	}

	*to = (int)lval;
	return(1);
}


int
parse_int(const char *from, int *to, int max, int min)
{
	long 		lval;
	char		*ep;

	assert(NULL != from);
	assert(NULL != to);
	
	errno = 0;
	lval = strtol(from, &ep, 10);
	if (from[0] == '\0' || *ep != '\0') {
		return(0);
	}
	if ((errno == ERANGE && (lval == LONG_MAX || lval == LONG_MIN)) 
			|| (lval > max || lval < min))
		return(0);

	*to = (int)lval;
	return(1);
}


/*
* Copyright (c) 2005, Nate Nielsen
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*     * Redistributions of source code must retain the above
*       copyright notice, this list of conditions and the
*       following disclaimer.
*     * Redistributions in binary form must reproduce the
*       above copyright notice, this list of conditions and
*       the following disclaimer in the documentation and/or
*       other materials provided with the distribution.
*     * The names of contributors to this software may not be
*       used to endorse or promote products derived from this
*       software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
* FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
* COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
* AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*
* CONTRIBUTORS
*  Nate Nielsen <nielsen@memberwebs.com>
*/
int
mkdir_p(char *path, mode_t mode)
{
	struct stat 	st;
	int 		last, retval;
	char		*p;

	p = path;
	retval = 0;
	
	/* Skip leading '/'. */
	while(p[0] == '/')
		++p;
	
	for(last = 0; ! last ; ++p) {
		if(p[0] == '\0')
			last = 1;
		else if (p[0] != '/')
			continue;
		*p = '\0';
		if(!last && p[1] == '\0')
			last = 1;
	
		/* Modified by the umask */
		if(mkdir(path, mode) < 0) {
			if(errno == EEXIST || errno == EISDIR) {
				if(stat(path, &st) < 0) {
					retval = 1;
					break;
				} else if(!S_ISDIR(st.st_mode)) {
					if (last)
						errno = EEXIST;
					else
						errno = ENOTDIR;
					retval = 1;
					break;
				}
			} else {
				retval = 1;
				break;
			}
		}
		if (!last)
			*p = '/';
	}

	return (retval);
}
